dofile_once("data/scripts/lib/utilities.lua")

local entity_id = GetUpdatedEntityID()
local x, y, rot, scale_x = EntityGetTransform(entity_id)
local one_if_looking_left = scale_x < 0 and 1 or 0

local autonomous = false

-- Keep these in sync with the ones in gui.lua (too lazy to extract them right now)
local action_ids = {
  ["climber"] = 1,
  ["umbrella"] = 2,
  ["exploder"] = 3,
  ["blocker"] = 4,
  ["builder"] = 5,
  ["digger_straight"] = 6,
  ["miner"] = 7,
  ["digger_down"] = 8,
}

function pixels_ahead(num_pixels)
  return num_pixels * scale_x - one_if_looking_left
end

SetRandomSeed(entity_id, entity_id)

function set_animation(anim)
  local sprite_comp = EntityGetFirstComponentIncludingDisabled(entity_id, "SpriteComponent")
  ComponentSetValue2(sprite_comp, "rect_animation", anim)
  EntityRefreshSprite(entity_id, sprite_comp)
end

function fix_countdown_text_position()
  for i, child in ipairs(EntityGetAllChildren(entity_id) or {}) do
    if EntityGetName(child) == "counter" then
      local counter_sprite_comp = EntityGetFirstComponentIncludingDisabled(child, "SpriteComponent")
      if counter_sprite_comp then
        ComponentSetValue2(counter_sprite_comp, "offset_x", scale_x > 0 and 2 or 5)
        EntityRefreshSprite(child, counter_sprite_comp)
        return
      end
    end
  end
end

fix_countdown_text_position()

function set_countdown_text(counter)
  for i, child in ipairs(EntityGetAllChildren(entity_id) or {}) do
    if EntityGetName(child) == "counter" then
      local counter_sprite_comp = EntityGetFirstComponentIncludingDisabled(child, "SpriteComponent")
      if counter_sprite_comp then
        ComponentSetValue2(counter_sprite_comp, "text", tostring(counter))
        EntityRefreshSprite(child, counter_sprite_comp)
        return
      end
    end
  end
end

local exit_door_tag = "cube_fx"
local blocker_tag = "monk_hand"
local build_chance = 0 --50
local blocking_chance = 0 --20
local dig_straight_chance = 0 --50
local dig_down_chance = 0 --50
local umbrella_chance = 0 --50
local fall_damage_height = 40

function should_build()
  local build_check_cooldown = GetValueInteger("build_check_cooldown", 0)
  if build_check_cooldown <= 0 then
    SetValueInteger("build_check_cooldown", 20)
    return Random(1, 100) <= build_chance
  else
    SetValueInteger("build_check_cooldown", build_check_cooldown - 1)
    return false
  end
end

function should_block()
  SetRandomSeed(entity_id + GameGetFrameNum(), entity_id + x + y)
  return Random(1, 100) <= blocking_chance
end

function should_dig_straight()
  SetRandomSeed(entity_id + GameGetFrameNum(), entity_id + x + y)
  return Random(1, 100) <= dig_straight_chance
end

function should_dig_down()
  SetRandomSeed(entity_id + GameGetFrameNum(), entity_id + x + y)
  return Random(1, 100) <= dig_down_chance
end

function should_open_umbrella()
  return GetValueInteger("umbrella_started_opening_on_tick", -1) == -1 and Random(1, 100) <= umbrella_chance
end

STATE_WALKING = 0
STATE_BUILDING = 1
STATE_DIGGING_STRAIGHT = 2
STATE_DIGGING_DOWN = 3
STATE_FALLING = 4
STATE_FALL_DEATH = 5
STATE_BLOCKING = 6
STATE_ABOUT_TO_EXPLODE = 7
STATE_EXIT = 8
STATE_DIGGING_DIAGONAL = 9
STATE_CLIMBING = 10
STATE_CLIMB_OVER = 11

local state = GetValueInteger("state", STATE_FALLING)
local funcs

function change_state(new_state)
  funcs[state].exit()
  SetValueInteger("state", new_state)
  funcs[new_state].enter()
  SetValueInteger("tick", 0)
  SetValueInteger("total_ticks_in_current_state", 0)
  local state_names = {
    [STATE_WALKING] = "Walking",
    [STATE_BUILDING] = "Building",
    [STATE_DIGGING_STRAIGHT] = "Digging straight",
    [STATE_DIGGING_DOWN] = "Digging down",
    [STATE_FALLING] = "Falling",
    [STATE_FALL_DEATH] = "Fall death",
    [STATE_BLOCKING] = "Blocking",
    [STATE_ABOUT_TO_EXPLODE] = "Exploding",
    [STATE_EXIT] = "Exit",
    [STATE_DIGGING_DIAGONAL] = "Digging diagonal",
    [STATE_CLIMBING] = "Climbing",
    [STATE_CLIMB_OVER] = "Climb over",
  }
  -- GamePrint("Changing state to: " .. state_names[new_state])
  -- print("Changing state to: " .. state_names[new_state])
end

local lemmings_stairs_found = false
for _, child in ipairs(EntityGetAllChildren(entity_id) or {} ) do
  if EntityGetName(child) == "walk_ahead_stair_check" then
    local material_check_var_comp = EntityGetFirstComponentIncludingDisabled(child, "VariableStorageComponent")
    if ComponentGetValue2(material_check_var_comp, "value_bool") then
      lemmings_stairs_found = true
      SetValueInteger("last_frame_stairs_detected", GameGetFrameNum())
      break
    end
  end
end

funcs = {
  [STATE_WALKING] = {
    enter = function()
      set_animation("walk")
      SetValueInteger("tick_max", 16)
    end,
    exit = function()
    end,
    update = function(tick, total_ticks_in_current_state)
      local blockers = EntityGetInRadiusWithTag(x + 1 * scale_x, y, 5, blocker_tag)
      for i, blocker in ipairs(blockers) do
        local blocker_x, blocker_y = EntityGetTransform(blocker)
        if (scale_x > 0 and blocker_x > x) or (scale_x < 0 and blocker_x < x) then
          -- Turn around
          EntitySetTransform(entity_id, x, y, rot, scale_x * -1)
          return
        end
      end
      if total_ticks_in_current_state % 30 == 0 and should_dig_down() then
        -- Scan downwards to see if there's somewhere to dig down to
        for i=1, 5 do
          local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y + i * 10, x, y + (i+1) * 10)
          if not did_hit then
            return change_state(STATE_DIGGING_DOWN)
          end
        end
      end
      local highest_step_over_y = 999999999999999999
      local did_hit, hit_x, hit_y
      local dt = GameGetFrameNum() - GetValueInteger("last_frame_stairs_detected", 99999999)
      if dt > 20 then
        SetValueInteger("last_frame_stairs_detected", 99999999)
      end
      local look_upwards_amount = (dt >= 0 and dt <= 28) and 1 or 3
      -- Scan 3 pixels below the entity for ground to determine if we should fall or move the entity towards it
      for i=1, 3 do
        did_hit, hit_x, hit_y = RaytraceSurfaces(x + pixels_ahead(-2 + i), y - look_upwards_amount, x + pixels_ahead(-2 + i), y + 4)
        highest_step_over_y = math.min(highest_step_over_y, hit_y)
      end
      local distance_to_ground = highest_step_over_y - y
      if distance_to_ground > 3 then
        return change_state(STATE_FALLING)
      else
        local wall_ahead = false
        local wall_x = x
        -- RaytraceSurfaces does not detect lemming stairs
        -- RaytracePlatforms does detect them
        local Raytrace = lemmings_stairs_found and RaytracePlatforms or RaytraceSurfaces
        for i=1, 4 do
          local hit, wall_x_ = Raytrace(x, y - 3 - i, x + scale_x * 2, y - 3 - i)
          wall_ahead = wall_ahead or hit
          wall_x = wall_x_
        end
        if wall_ahead then
          -- Check if there is no ceiling so we can climb
          local any_hit = false
          for i=1, 3 do
            local did_hit, hit_x, hit_y = RaytraceSurfaces(x + pixels_ahead(-3 + i), y - 3, x + pixels_ahead(-3 + i), y - 6)
            any_hit = any_hit or did_hit
          end
          if GetValueBool("climber_mode_enabled", false) and not any_hit then
            EntitySetTransform(entity_id, wall_x, y)
            return change_state(STATE_CLIMBING)
          elseif should_dig_straight() then
            return change_state(STATE_DIGGING_STRAIGHT)
          else
            -- Turn around
            EntitySetTransform(entity_id, x, y, rot, scale_x * -1)
          end
        else
          -- Walk forward
          EntitySetTransform(entity_id, x + 1 * scale_x, math.floor(math.min(hit_y, highest_step_over_y)))
        end
      end
      local did_hit, hit_x, hit_y = RaytraceSurfaces(x + pixels_ahead(8), y, x + pixels_ahead(8), y + 30)
      if not did_hit then
        if should_build() then
          return change_state(STATE_BUILDING)
        elseif should_block() then
          return change_state(STATE_BLOCKING)
        end
      end
    end,
  },
  [STATE_BUILDING] = {
    enter = function()
      set_animation("build")
      SetValueBool("build_move_to_new_position", false)
      SetValueInteger("tick_max", 16)
    end,
    exit = function()
    end,
    update = function(tick, total_ticks_in_current_state)
      local T = 1
      local move_to_new_position = GetValueBool("build_move_to_new_position", false)
      if move_to_new_position and tick == 1 then
        SetValueBool("build_move_to_new_position", false)
        EntitySetTransform(entity_id, x + 2 * scale_x, y - 1)
        set_animation("build")
      end
      if tick == 10 then
        local a = scale_x < 0 and -8 or 0
        GamePlaySound("mods/lemmings/audio/lemmings.bank", "TING", x, y)
        LoadPixelScene("mods/lemmings/files/platform.png", "mods/lemmings/files/platform_visual.png", x + 2 + entity_id / 10000000 + a, y, "", true, true)
      end
      if tick == 16 then
        SetValueBool("build_move_to_new_position", true)
        local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y, x + 4 * scale_x, y - 5)
        if did_hit then
          return change_state(STATE_WALKING)
        end
      end
      -- Check if ground was removed from below, if so, fall
      local did_hit, hit_x, hit_y = RaytraceSurfaces(x + 1 * scale_x, y, x + 1 * scale_x, y + 2)
      if not did_hit then
        return change_state(STATE_FALLING)
      end
      if autonomous then
        -- Check if there is ground ahead to jump down to, if so, stop building and start walking
        local did_hit, hit_x, hit_y = RaytraceSurfaces(x + 2 * scale_x, y + 2, x + 2 * scale_x, y + 29)
        if total_ticks_in_current_state > 60 and did_hit then
          return change_state(STATE_WALKING)
        end
      elseif total_ticks_in_current_state > 16 * 15 then
        return change_state(STATE_WALKING)
      end
    end,
  },
  [STATE_DIGGING_STRAIGHT] = {
    enter = function()
      set_animation("digging_straight")
      SetValueInteger("tick_max", 32)
    end,
    exit = function()
    end,
    update = function(tick, total_ticks_in_current_state)
      if tick == 1 then
        set_animation("digging_straight")
      end
      -- Scan 4 pixels below the entity for ground to determine if we should fall
      local any_hit = false
      for i=1, 4 do
        local did_hit, hit_x, hit_y = RaytraceSurfaces(x + (-1 + i) * scale_x, y - 1, x + (-1 + i) * scale_x, y + 2)
        any_hit = any_hit or did_hit
      end
      if not any_hit then
        return change_state(STATE_FALLING)
      end
      if (total_ticks_in_current_state > 1 and tick == 1) or tick == 17 then
        local x, y, rot, scale_x = EntityGetTransform(entity_id)
        EntitySetTransform(entity_id, x + 4 * scale_x, y)
      end
      local add_one = scale_x < 0 and -1 or 0
      EntityLoad("mods/lemmings/files/digger.xml", x + math.floor((tick % 16) / 4) * scale_x + add_one, y - 5)
      local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y, x + 10 * scale_x, y)
      if not did_hit then
        return change_state(STATE_WALKING)
      end
    end,
  },
  [STATE_DIGGING_DOWN] = {
    enter = function()
      set_animation("digging_down")
      SetValueInteger("tick_max", 16)
      SetValueInteger("digging_down_fall_at_tick", -1)
    end,
    exit = function()
    end,
    update = function(tick, total_ticks_in_current_state)
      if GetValueInteger("digging_down_fall_at_tick", -1) == total_ticks_in_current_state then
        return change_state(STATE_FALLING)
      end
      if tick == 1 then
        set_animation("digging_down")
      end
      local x, y, rot, scale_x = EntityGetTransform(entity_id)
      if tick % 3 == 0 then
        EntityLoad("mods/lemmings/files/digger.xml", x, y - 4)
        EntitySetTransform(entity_id, x, y + 1)
      end
      local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y + 1, x, y + 3)
      if not did_hit and GetValueInteger("digging_down_fall_at_tick", -1) == -1 then
        SetValueInteger("digging_down_fall_at_tick", total_ticks_in_current_state + 10)
      end
    end,
  },
  [STATE_FALLING] = {
    enter = function()
      set_animation("falling")
      SetValueInteger("tick_max", 4)
      SetValueBool("umbrella_is_open", false)
      SetValueInteger("umbrella_started_opening_on_tick", -1)
    end,
    exit = function()
    end,
    update = function(tick, total_ticks_in_current_state)
      local x, y, rot, scale_x = EntityGetTransform(entity_id)
      local umbrella_is_open = GetValueBool("umbrella_is_open", false)
      EntitySetTransform(entity_id, x, y + (umbrella_is_open and 1 or 2))
      local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y, x, y + 3)
      if did_hit then
        if total_ticks_in_current_state > fall_damage_height and not umbrella_is_open then
          GamePlaySound("mods/lemmings/audio/lemmings.bank", "SPLAT", x, y)
          return change_state(STATE_FALL_DEATH)
        else
          return change_state(STATE_WALKING)
        end
      end
      if total_ticks_in_current_state == fall_damage_height - 1 and should_open_umbrella() then
        set_animation("umbrella_open")
        SetValueInteger("umbrella_started_opening_on_tick", total_ticks_in_current_state)
      end
      local umbrella_started_opening_on_tick = GetValueInteger("umbrella_started_opening_on_tick", -1)
      if not GetValueBool("umbrella_is_open", false) and umbrella_started_opening_on_tick > -1 and umbrella_started_opening_on_tick + 9 == total_ticks_in_current_state then
        set_animation("umbrella_float")
        SetValueBool("umbrella_is_open", true)
        SetValueInteger("umbrella_started_opening_on_tick", -1)
      end
    end,
  },
  [STATE_FALL_DEATH] = {
    enter = function()
      set_animation("fall_death")
      SetValueInteger("tick_max", 7)
    end,
    exit = function()
    end,
    update = function(tick, total_ticks_in_current_state)
      if tick == 7 then
        EntityKill(entity_id)
      end
    end,
  },
  [STATE_BLOCKING] = {
    enter = function()
      set_animation("blocking")
      SetValueInteger("tick_max", 16)
      -- Add an existing tag to identify blockers so we don't create a new one because of tag limit of 256
      if not EntityHasTag(entity_id, blocker_tag) then
        EntityAddTag(entity_id, blocker_tag)
      end
    end,
    exit = function()
      EntityRemoveTag(entity_id, blocker_tag)
    end,
    update = function(tick, total_ticks_in_current_state)
    end,
  },
  [STATE_ABOUT_TO_EXPLODE] = {
    enter = function()
      set_animation("about_to_explode_1")
      SetValueInteger("tick_max", 9)
      GamePlaySound("mods/lemmings/audio/lemmings.bank", "OHNO", x, y)
    end,
    exit = function()
    end,
    update = function(tick, total_ticks_in_current_state)
      if total_ticks_in_current_state == 6 then
        set_animation("about_to_explode_2")
      end
      if total_ticks_in_current_state == 18 then
        set_animation("explosion")
        EntityLoad("mods/lemmings/files/explosion_digger.xml", x, y)
        EntityLoad("mods/lemmings/files/explosion_digger_particles.xml", x, y)
        GamePlaySound("mods/lemmings/audio/lemmings.bank", "EXPLODE", x, y)
        EntitySetTransform(entity_id, x, y + 3)
      end
      if total_ticks_in_current_state == 23 then
        EntityKill(entity_id)
      end
    end,
  },
  [STATE_EXIT] = {
    enter = function()
      SetValueInteger("tick_max", 8)
      set_animation("exit")
      GamePlaySound("mods/lemmings/audio/lemmings.bank", "YIPPEE", x, y)
    end,
    exit = function()
    end,
    update = function(tick, total_ticks_in_current_state)
      if total_ticks_in_current_state == 1 then
        local exit_door = EntityGetInRadiusWithTag(x, y, 2, exit_door_tag)[1]
        local x, y = EntityGetTransform(exit_door)
        EntitySetTransform(entity_id, x - 1, y - 1)
      end
      if total_ticks_in_current_state == 8 then
        EntityKill(entity_id)
        local exit_door = EntityGetInRadiusWithTag(x, y, 2, exit_door_tag)[1]
        if exit_door then
          local var_store = EntityGetFirstComponentIncludingDisabled(exit_door, "VariableStorageComponent")
          local entered = ComponentGetValue2(var_store, "value_int")
          ComponentSetValue2(var_store, "value_int", entered + 1)
        end
      end
    end,
  },
  [STATE_DIGGING_DIAGONAL] = {
    enter = function()
      SetValueInteger("tick_max", 24)
      set_animation("digging_diagonal")
    end,
    exit = function()
    end,
    update = function(tick, total_ticks_in_current_state)
      local x, y = EntityGetTransform(entity_id)
      if tick == 3 then
        EntityLoad("mods/lemmings/files/digger.xml", x + pixels_ahead(5), y - 3)
        GamePlaySound("mods/lemmings/audio/lemmings.bank", "CHINK", x, y)
      end
      if tick == 4 then
        EntitySetTransform(entity_id, x + 2 * scale_x, y + 1)
      end
      if tick == 8 then
        EntitySetTransform(entity_id, x, y + 1)
      end
      if tick == 15 then
        EntitySetTransform(entity_id, x + 2 * scale_x, y)
      end
      if tick == 24 then
        -- Check if we're done digging
        local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y, x + pixels_ahead(4), y + 2)
        if not did_hit then
          return change_state(STATE_WALKING)
        end
        set_animation("digging_diagonal")
      end
      -- Scan 4 pixels below the entity for ground to determine if we should fall
      local any_hit = false
      for i=1, 4 do
        local did_hit, hit_x, hit_y = RaytraceSurfaces(x + (-1 + i) * scale_x, y - 1, x + (-1 + i) * scale_x, y + 2)
        any_hit = any_hit or did_hit
      end
      if not any_hit then
        return change_state(STATE_FALLING)
      end
    end,
  },
  [STATE_CLIMBING] = {
    enter = function()
      SetValueInteger("tick_max", 8)
      set_animation("climbing")
    end,
    exit = function()
    end,
    update = function(tick, total_ticks_in_current_state)
      local x, y = EntityGetTransform(entity_id)
      -- Scan 4 pixels above the entity to detect edge to climb over
      local should_climb = true
      -- If there's 2 pixels in a row missing, stop climbing
      local last_hit = true
      local hit_positions_x = {}
      for i=1, 5 do
        local did_hit, hit_x, hit_y = RaytraceSurfaces(x - pixels_ahead(2), y - i, x + pixels_ahead(2), y - i)
        -- If we missed two in a row, fall
        if not (last_hit or did_hit) then
          should_climb = false
          break
        end
        last_hit = did_hit
      end
      local any_hit = false
      for i=1, 4 do
        local did_hit, hit_x, hit_y = RaytraceSurfaces(x, y - 7 - i, x + pixels_ahead(4), y - 7 - i)
        any_hit = any_hit or did_hit
      end
      -- Check if there's ceiling, if so, fall
      local did_hit_ceiling, hit_x, hit_y = RaytraceSurfaces(x - pixels_ahead(3), y - 6, x - pixels_ahead(3), y - 8)
      if not any_hit and total_ticks_in_current_state > 1 then
        return change_state(STATE_CLIMB_OVER)
      elseif did_hit_ceiling or not should_climb then
        EntitySetTransform(entity_id, x - pixels_ahead(1), y, 0, scale_x * -1)
        return change_state(STATE_FALLING)
      end
      local did_hit, hit_x, hit_y = RaytraceSurfaces(x - pixels_ahead(2), y - 3, x + pixels_ahead(2), y - 3)
      if tick == 3 or tick == 4 or tick == 5 then
        EntitySetTransform(entity_id, hit_x + one_if_looking_left, y - 1)
      end
    end,
  },
  [STATE_CLIMB_OVER] = {
    enter = function()
      SetValueInteger("tick_max", 8)
      set_animation("climb_over")
    end,
    exit = function()
    end,
    update = function(tick, total_ticks_in_current_state)
      if total_ticks_in_current_state == 9 then
        local x, y = EntityGetTransform(entity_id)
        EntitySetTransform(entity_id, x + 2 * scale_x, y - 8)
        return change_state(STATE_WALKING)
      end
    end,
  },
}

local tick = tonumber(GetValueInteger("tick", 1))
local total_ticks_in_current_state = tonumber(GetValueInteger("total_ticks_in_current_state", 1))

for i, comp in ipairs(EntityGetComponentIncludingDisabled(entity_id, "VariableStorageComponent") or {}) do
  if ComponentGetValue2(comp, "name") == "command" then
    local command = ComponentGetValue2(comp, "value_string")
    if command == "climber" then
      if not GetValueBool("climber_mode_enabled", false) then
        SetValueBool("climber_mode_enabled", true)
        GlobalsSetValue("lemmings_action_id_performed", action_ids.climber)
      end
    elseif command == "umbrella" then
      if state == STATE_FALLING and GetValueInteger("umbrella_started_opening_on_tick", -1) == -1 and not GetValueBool("umbrella_is_open", false) then
        set_animation("umbrella_open")
        SetValueInteger("umbrella_started_opening_on_tick", total_ticks_in_current_state)
        GlobalsSetValue("lemmings_action_id_performed", action_ids.umbrella)
      end
    elseif command == "builder" then
      if state == STATE_WALKING then
        change_state(STATE_BUILDING)
        GlobalsSetValue("lemmings_action_id_performed", action_ids.builder)
      end
    elseif command == "digger_straight" then
      if state == STATE_WALKING then
        change_state(STATE_DIGGING_STRAIGHT)
        GlobalsSetValue("lemmings_action_id_performed", action_ids.digger_straight)
      end
    elseif command == "blocker" then
      if state == STATE_WALKING then
        change_state(STATE_BLOCKING)
        GlobalsSetValue("lemmings_action_id_performed", action_ids.blocker)
      end
    elseif command == "digger_down" then
      if state == STATE_WALKING then
        change_state(STATE_DIGGING_DOWN)
        GlobalsSetValue("lemmings_action_id_performed", action_ids.digger_down)
      end
    elseif command == "exploder" then
      if GetValueInteger("explode_frame_counter", -1) < 0 then
        SetValueInteger("explode_frame_counter", 0)
        set_countdown_text(5)
        GlobalsSetValue("lemmings_action_id_performed", action_ids.exploder)
      end
    elseif command == "miner" then
      if state == STATE_WALKING then
        change_state(STATE_DIGGING_DIAGONAL)
        GlobalsSetValue("lemmings_action_id_performed", action_ids.miner)
      end
    end
    ComponentSetValue2(comp, "value_string", "")
  end
end

local exit_doors = EntityGetInRadiusWithTag(x, y, 2, exit_door_tag)
if state ~= STATE_EXIT and #exit_doors > 0 then
  change_state(STATE_EXIT)
end
funcs[state].update(tick, total_ticks_in_current_state)
tick = tonumber(GetValueInteger("tick", 1))
local tick_max = GetValueInteger("tick_max", 16)
-- tick goes from 1 to tick_max
SetValueInteger("tick", math.max(1, (tick + 1) % (tick_max + 1)))
SetValueInteger("total_ticks_in_current_state", tonumber(GetValueInteger("total_ticks_in_current_state", 1)) + 1)

local damage_model_comp = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")

if ComponentGetValue2(damage_model_comp, "mAirAreWeInWater") and GetValueInteger("last_frame_glug_played", 0) + 120 < GameGetFrameNum() then
  SetValueInteger("last_frame_glug_played", GameGetFrameNum())
  GamePlaySound("mods/lemmings/audio/lemmings.bank", "GLUG", x, y)
end

local counter = GetValueInteger("explode_frame_counter", -1)
if counter >= 0 and counter <= 75 then
  counter = counter + 1
  SetValueInteger("explode_frame_counter", counter)
  if counter % 15 == 0 then
    if counter == 75 then
      set_countdown_text("")
      return change_state(STATE_ABOUT_TO_EXPLODE)
    else
      set_countdown_text(5 - math.floor(counter / 15))
    end
  end
end
